//+------------------------------------------------------------------+
//|                                                SupremacyTest.mq5 |
//|                                         Copyright 2015, nicholas |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, nicholas"
#property link      ""
#property version   "1.02"
#include <Arrays\List.mqh>
#include <Trade\Trade.mqh>
#include <Trade\SymbolInfo.mqh>
#include <Trade\PositionInfo.mqh>
#include <Trade\AccountInfo.mqh>

input string   Pairs                      = "EURUSD,USDCHF,USDCAD,GBPUSD,AUDUSD,NZDUSD,EURJPY,AUDJPY,USDJPY,GBPAUD,GBPCAD,GBPCHF,GBPJPY,GBPNZD,XAUUSD";
//input string   TerminalPairs              = "EURUSD.m,USDCHF.m,USDCAD.m,GBPUSD.m,AUDUSD.m,NZDUSD.m,EURJPY.m,AUDJPY.m,USDJPY.m,GBPAUD.m,GBPCAD.m,GBPCHF.m,GBPJPY.m,GBPNZD.m,XAUUSD.m";
input string   TerminalPairs              = "";
input string   PairsSufix                 = ".m";
input int      TimeFrame                  = 1440;
input int      Timeout                    = 10000; //  .  1000 (1 .)     
input int      MaxOpenPercent             = 40;

input string   _order                     = "===  ===";
//input int      MMMethod                   = 1;        // 0 -    (Lots), 1 -   
input double   Lots                       = 0.1;     //    MinDepo  MMMethod=0     MMMethod=1
//input double   MinDepo                    = 1000;     //       MMMethod=1.   ( ).        DepoStep     LotsStep
//input double   DepoStep                   = 1000;     //       LotsStep
//input double   LotsStep                   = 1;     //  
input int      MarketStopLoss             = 0;
//extern int     MinLimitOrderDistance      = 10;
//extern int     LimitStopLoss              = 50;
input int      TakeProfit                 = 0;
//input bool     OpenLimitOrders            = true;

input string   _other                     = "===   ===";
input int      Magic                      = 342442;   // .             .
input int      Slippage                   = 3;        //  
input bool     Auto5Digits                = true;     //   5-  .         10    5-  
input string   EaComment                  = "SupremacyTest v1.02"; //  

//+------------------------------------------------------------------+
const string MyFxBookDataUrl = "http://www.myfxbook.com/symbolOutlookConfig.json";
const string ParametersFormat = "?type=0&symbolOid=%d&timeFrame=%d&rand=%f";

class MyFxBookData: CObject
{
private:      
   string         _symbol;
   int            _oid;
   string         _workUrl;
   int            _timeFrame;
   bool           _useCache;
   datetime       _percentagesTimes[];
   int            _shortPercentages[];
   CTrade         _trade;                      // trading object
   CPositionInfo  _position;                   // trade position object
   CSymbolInfo    _symbolInfo;                     // symbol info object
   CAccountInfo   _account;                    // account info wrapper
   double         _adjustedPoint;             // point value adjusted for 3 or 5 points
   double         _takeProfit;
   double         _stopLoss;
private:
   void InitHistory();
   int GetOid();
   bool DownloadData(char &result[]);
   int GetLastShortPercent();
public:  
   MyFxBookData(string symbol, int oid);
   void Processing();
};
//+------------------------------------------------------------------+

CList pairsList;

int OnInit()
{
   Print("Pairs='", Pairs, "'");
   Print(StringLen(Pairs));
   Print("TerminalPairs='", TerminalPairs, "'");
   Print(StringLen(TerminalPairs));
   
   string tempPairs[];
   string tempTerminalPairs[];
   StringSplit(Pairs, ',', tempPairs);
   if(TerminalPairs != "")
   {
      StringSplit(TerminalPairs, ',', tempTerminalPairs);
   }
   else
   {
      StringSplit(Pairs, ',', tempTerminalPairs);
   }
   
   Print(ArraySize(tempPairs));
   Print(ArraySize(tempTerminalPairs));
   if(ArraySize(tempPairs) != ArraySize(tempTerminalPairs))
   {
      Print("Error: number of items in TermainPairs parameter should match number of items of Paris");
      return (INIT_PARAMETERS_INCORRECT);
   }      
   
   for(int i=0; i < ArraySize(tempPairs); i++)
   {
      StringToUpper(tempPairs[i]);
      StringTrimLeft(tempPairs[i]);
      StringTrimRight(tempPairs[i]);      
      int oid = GetOid(tempPairs[i]);
      if(oid < 0)
      {
         Print("Pair ", tempPairs[i], " doesn't exists in myfxbook");
         continue;
      }
      
      string termainalPair;
      StringTrimLeft(tempTerminalPairs[i]);
      StringTrimRight(tempTerminalPairs[i]); 
      StringConcatenate(termainalPair, tempTerminalPairs[i], PairsSufix);
      
      if(!IsRealSymbol(termainalPair))
      {
         Print("Pair ", termainalPair, " doesn't exists in the terminal");
         continue;
      }
      
      MarketBookAdd(termainalPair);
      pairsList.Add(new MyFxBookData(termainalPair, oid));
   }
    
   return(INIT_SUCCEEDED);
}

void OnDeinit(const int reason)
{
}

void OnTick()
{
   if(!IsNewBar())
      return;
      
   for(int i=0; i < pairsList.Total(); i++)
   {
      MyFxBookData *pair = pairsList.GetNodeAtIndex(i);
      pair.Processing();
   }
}

//+------------------------------------------------------------------+
bool IsNewBar() {
	static datetime lastCheckTime;
	datetime lastBarTime = SeriesInfoInteger(Symbol(),Period(),SERIES_LASTBAR_DATE);
	if (lastCheckTime == lastBarTime) 
		return (false);
	lastCheckTime = lastBarTime;
	return (true);
}
//+------------------------------------------------------------------+
bool IsRealSymbol(string symbol)
{
   for(int s=0; s<SymbolsTotal(false); s++)
   {
      string name = SymbolName(s,false);
      if(symbol == SymbolName(s,false))
         return(true);
   }
   
   return(false);
}
  
//+------------------------------------------------------------------+
MyFxBookData::MyFxBookData(string symbol, int oid):
    _symbol(symbol), _oid(oid), _timeFrame(1440)
{
   _useCache = true;
   _trade.SetExpertMagicNumber(Magic);
   _symbolInfo.Name(_symbol);
   
   int digitsAdjust=1;
   if(_symbolInfo.Digits() == 3 || _symbolInfo.Digits() == 5)
      digitsAdjust=10;
      
   _adjustedPoint = _symbolInfo.Point()*digitsAdjust;
   _takeProfit = TakeProfit*_adjustedPoint;
   _stopLoss = MarketStopLoss*_adjustedPoint;


   _trade.SetDeviationInPoints(Slippage*digitsAdjust);
   
   InitHistory();
}

//+------------------------------------------------------------------+
void MyFxBookData::Processing(void)
{
   //--- refresh rates
   if(!_symbolInfo.RefreshRates())
      return;
      
   if(_position.Select(_symbol))
   {
      if(_trade.PositionClose(_symbol))
         printf("Position by %s to be closed", _symbol);
      else
         printf("Error closing position by %s : '%s'", _symbol, _trade.ResultComment());
   }
      
   int shortPercent = GetLastShortPercent();
   if(shortPercent < 0)
      return;
      
   Print("Symbol ", _symbol, " shortPercent=", shortPercent);
   
   if(shortPercent <= MaxOpenPercent)
   {
      double price = _symbolInfo.Bid();
      double tp=0.0, sl=0.0;
      
      if(_takeProfit > 0.0)
         tp = _symbolInfo.Ask() - _takeProfit;
      
      if(_stopLoss > 0.0)
         sl = _symbolInfo.Ask() + _stopLoss;
      
      //--- check for free money
      Print("Lots=", Lots);
      Print("price=", price);
      if(_account.FreeMarginCheck(Symbol(), ORDER_TYPE_SELL, Lots, price) < 0.0)
         printf("We have no money. Free Margin = %f",_account.FreeMargin());
      else
      {
         //--- open position
         if(_trade.PositionOpen(_symbol, ORDER_TYPE_SELL, Lots, price, sl, tp, EaComment))
            printf("Position by %s to be opened", _symbol);
         else
         {
            printf("Error opening SELL position by %s : '%s'", _symbol, _trade.ResultComment());
            printf("Open parameters : price=%f,TP=%f", price, tp);
         }
      }         

   } 
   else if(shortPercent >= (100-MaxOpenPercent))
   {
      double price = _symbolInfo.Ask();
      double tp = 0.0, sl = 0.0;
      
      if(_takeProfit > 0.0)
         tp = _symbolInfo.Bid() + _takeProfit;
      if(_stopLoss  > 0.0)
         sl = _symbolInfo.Bid() - _stopLoss;
      //--- check for free money
      if(_account.FreeMarginCheck(Symbol(), ORDER_TYPE_BUY, Lots, price) < 0.0)
         printf("We have no money. Free Margin = %f",_account.FreeMargin());
      else
      {
         //--- open position
         if(_trade.PositionOpen(_symbol, ORDER_TYPE_BUY, Lots, price, sl, tp, EaComment))
            printf("Position by %s to be opened", _symbol);
         else
         {
            printf("Error opening BUY position by %s : '%s'", _symbol, _trade.ResultComment());
            printf("Open parameters : price=%f,TP=%f", price, tp);
         }
      }              
   }  
}
//+------------------------------------------------------------------+
void MyFxBookData::InitHistory(void)
{
   string shortsStr, longsStr;
   string fileName = _symbol + "_"+IntegerToString(_timeFrame)+".data";
   Print("Check file: ", fileName);   
   //if(!FileIsExist(fileName, 1) || !_useCache)
   if(!MQLInfoInteger(MQL_TESTER))
   {
      MathSrand(GetTickCount());
      _workUrl = MyFxBookDataUrl+StringFormat(ParametersFormat, _oid, _timeFrame, (1.0/(1.0+MathRand())));
      Print("Download data from the url=", _workUrl);
      
      uint start=GetTickCount();
      char html[];
      if(!DownloadData(html))
      {         
         Print("Can't download data from myfxbook using:", _workUrl);
         return;
      }
      
      uint downloadTime=GetTickCount()-start;
      PrintFormat("Downloading mysqldata took %d ms", downloadTime);
      
      int fh = FileOpen(fileName, FILE_WRITE|FILE_COMMON);         
      int startFrom = 0;
      shortsStr = GetDataString(html, startFrom);            
      longsStr = GetDataString(html, startFrom);
      
      FileWrite(fh, shortsStr);
      FileWrite(fh, longsStr);     
      FileClose(fh);         
   }
   else
   {
      int fh = FileOpen(fileName, FILE_READ|FILE_TXT|FILE_COMMON);
      shortsStr = FileReadString(fh);
      longsStr  = FileReadString(fh);
      FileClose(fh);
   }
   
    string shortItems[], longItems[];
   if(StringSplit(shortsStr, ';', shortItems) <= 0)
   {
      Print("Error: can't split shorts percentages string");
      return;
   }
   if(StringSplit(longsStr, ';', longItems) <= 0)   
   {
      Print("Error: can't split longs percentages string");
      return;
   }     
   
   int size = ArraySize(shortItems);
   while(size)
   {
      if(shortItems[size-1] != "")
         break;
      size--;
   }
   if(size != ArraySize(shortItems))
   {
      ArrayResize(shortItems, size);
      ArrayResize(longItems, size);
   }
   
   if(size != ArraySize(longItems))
   {
      Print("Error: number of shorts (%d) doesn't equals to number of longs (%d)", size, ArraySize(longItems));
      return;
   }
   
   ArrayResize(_percentagesTimes, size);
   ArrayResize(_shortPercentages, size);   
   for(int i=0; i< size; i++)
   {
      if(shortItems[i] == "")
         continue;
         
      string shortParts[], longParts[];
      if(StringSplit(shortItems[i], ',', shortParts) < 2)
      {
         Print("Error: can't split shorts percent item: ", i, " \"", shortItems[i], "\"");
         return;
      }
      if(StringSplit(longItems[i], ',', longParts) < 2)   
      {
         Print("Error: can't split long percent item: ", i, " \"", longItems[i], "\"");
         return;
      }
      
      datetime shortTime = StringToInteger(shortParts[0])/1000;
      datetime longTime = StringToInteger(shortParts[0])/1000;
      if(shortTime != longTime)
      {
         Print("Error: short and long times don't match");
         return;
      }
      _percentagesTimes[i] = shortTime;
      double shortPositions = StringToDouble(shortParts[1]);
      double longPositions = StringToDouble(longParts[1]);
      if(shortPositions <= 0.0 && longPositions <= 0.0)
      {
         Print("Error: incorect positions size");
         return;
      }
      _shortPercentages[i] = MathRound(shortPositions/(shortPositions+longPositions)*100.0);
   }      
}
//+------------------------------------------------------------------+
int MyFxBookData::GetLastShortPercent()
{
   if(ArraySize(_percentagesTimes) <= 0)
      return -1;
      
   if(TimeCurrent() <  _percentagesTimes[0])
      return -1;
       
   for(int i=1; i < ArraySize(_shortPercentages); i++)
   {
      if(TimeCurrent() >= _percentagesTimes[i-1] && TimeCurrent() < _percentagesTimes[i])
         return _shortPercentages[i-1];
   }
   
   return -1;
}

//+------------------------------------------------------------------+
bool MyFxBookData::DownloadData(char &result[])
{
   string resultHeaders;
   char post[];
   
   ResetLastError();   
   
   if(WebRequest("GET", _workUrl, NULL, NULL, Timeout, post, 0, result, resultHeaders) < 0)
   {
      Print("  WebRequest.    =",GetLastError());      
      return false;
   }
        
   return true;
}

//+------------------------------------------------------------------+
string GetDataString(const char& charArray[], int& startFrom)
{
   int startPos = StringFindInArray(charArray, "<![CDATA[", startFrom, 2);
   if(startPos < 0)
   {
      Print("Error: can't find info for pair");
      return "";
   }
   
   startPos+=9;
   
   int endPos = StringFindInArray(charArray, "]]>", startPos);
   if(endPos < 0)
   {
      Print("Error: can't find info for pair");
      return "";
   }
   
   startFrom = endPos +1;   
   
   return CharArrayToString(charArray, startPos, endPos - startPos);
}

//+------------------------------------------------------------------+
int StringFindInArray(const char& arrayValue[], string matchSubstring, int startPos = 0, int count=1)
{
   char matchSubstringArray[];
   //StringToCharArray(matchSubstring, matchSubstringArray);

   for(int i=startPos; i < ArraySize(arrayValue); i++)
   {
      if(arrayValue[i] == StringGetCharacter(matchSubstring, 0))
      {
         bool found = true;
         for(int j = 1; j < StringLen(matchSubstring) && found; j++)
         {
            if(arrayValue[i+j] != StringGetCharacter(matchSubstring, j))
               found = false;
         }  
         
         if(found)
         {
            count--;            
            if(count<=0)            
               return i;
         }
      }
   }
   
   return -1;
}

int GetOid(string symbol)
{
   static int symbolOids[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 17, 16, 19, 21, 20, 23, 22, 25, 24, 27, 26, 29, 28, 
      31, 32, 33, 38, 36, 37, 42, 43, 40, 41, 46, 47, 45, 51, 50, 49, 48, 103, 107, 137, 131, 1236, 1233, 1234, 1245, 1246, 1247, 1253, 
      1863, 2012, 2076, 2090, 2119, 3240, 3473, 5079, 5435, 5779, 10064, 12755};
      
   static string symbolNames[] = {"EURUSD", "GBPUSD", "USDJPY", "GBPJPY", "USDCAD", "EURAUD", "EURJPY", "AUDCAD", "AUDJPY", "AUDNZD", 
      "AUDUSD", "CADJPY", "EURCAD", "EURCHF", "EURCZK", "EURGBP", "EURDKK", "EURNOK", "EURPLN", "EURNZD", "EURTRY", "EURSEK", "GBPCHF", 
      "GBPCAD", "NZDJPY", "NZDCAD", "USDCHF", "NZDUSD", "USDCZK", "USDDKK", "USDHKD", "USDPLN", "USDMXN", "USDNOK", "USDTHB", "USDTRY", 
      "USDSEK", "USDSGD", "CHFJPY", "AUDCHF", "USDZAR", "XAUUSD", "XAGUSD", "NZDCHF", "GBPNZD", "CADCHF", "GBPAUD", "GBPNOK", "SGDJPY", 
      "GBPSEK", "EURZAR", "AUDSGD", "CHFSGD", "EURSGD", "GBPSGD", "GBPDKK", "NOKJPY", "GBPTRY", "USDRUB", "SEKJPY", "XAUEUR", "NOKSEK", 
      "XAGEUR", "XPTUSD", "XPDUSD", "USDCNH", "XTIUSD", "XBRUSD"
   };
            
   for(int i=0; i < ArraySize(symbolNames); i++)
   {
      if(StringCompare(symbol, symbolNames[i], false)==0)
      {
         return symbolOids[i];
      }
   }
   return -1;
}

